package com.tzld.longarticle.recommend.server.config.db;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.boot.orm.jpa.EntityManagerFactoryBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import javax.persistence.EntityManagerFactory;
import javax.sql.DataSource;
import java.util.HashMap;
import java.util.Map;

@Configuration
@EnableTransactionManagement
@EnableJpaRepositories(
        basePackages = "com.tzld.longarticle.recommend.server.repository.adplatform",
        entityManagerFactoryRef = "adplatformEntityManagerFactory",
        transactionManagerRef = "adplatformTransactionManager"
)
public class AdplatformDBConfig {

    @Value("${spring.jpa.adplatform.hibernate.ddl-auto}")
    private String ddlAuto;
    @Value("${spring.jpa.adplatform.database}")
    private String database;

    public Map<String, Object> hibernateProperties() {
        Map<String, Object> properties = new HashMap<>();
        properties.put("hibernate.ddl-auto", ddlAuto);
        properties.put("database", database);
        return properties;
    }

    @Bean(name = "adplatformDataSource")
    @ConfigurationProperties(prefix = "spring.datasource.adplatform")
    public DataSource adplatformDataSource() {
        return DataSourceBuilder.create().build();
    }

    @Bean(name = "adplatformEntityManagerFactory")
    public LocalContainerEntityManagerFactoryBean adplatformEntityManagerFactory(EntityManagerFactoryBuilder builder,
                                                                                 @Qualifier("adplatformDataSource") DataSource dataSource) {
        return builder
                .dataSource(dataSource)
                .packages("com.tzld.longarticle.recommend.server.repository.entity.adplatform") // 实体类包路径
                .persistenceUnit("adplatform")
                .properties(hibernateProperties())
                .build();
    }

    @Bean(name = "adplatformTransactionManager")
    public PlatformTransactionManager adplatformTransactionManager(
            @Qualifier("adplatformEntityManagerFactory") EntityManagerFactory adplatformEntityManagerFactory) {
        return new JpaTransactionManager(adplatformEntityManagerFactory);
    }
}

