package com.tzld.longarticle.recommend.server.remote;

import com.aliyun.odps.utils.StringUtils;
import com.google.common.reflect.TypeToken;
import com.tzld.longarticle.recommend.server.common.HttpPoolFactory;
import com.tzld.longarticle.recommend.server.model.WxUserGetResponse;
import com.tzld.longarticle.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * @author dyp
 */
@Service
@Slf4j
public class WxUserManagementRemoteService {

    private final CloseableHttpClient client = HttpPoolFactory.defaultPool();

    @Autowired
    private WxAccessTokenRemoteService wxAccessTokenRemoteService;

    public WxUserGetResponse getAllUserLimit10000(String gzhId) {

        try {

            String accessToken = wxAccessTokenRemoteService.getAccessToken(gzhId);
            if (StringUtils.isBlank(accessToken)) {
                log.error("gzh {} get access_token fail", gzhId);
                return null;
            } else {
                log.info("gzh {} access_token {}", gzhId, accessToken);
            }

            URIBuilder uriBuilder = new URIBuilder("https://api.weixin.qq.com/cgi-bin/user/get");
            uriBuilder.setParameter("gzhId", gzhId); // 假设需要传递gzhId参数
            uriBuilder.setParameter("access_token", accessToken);
            HttpGet httpGet = new HttpGet();
            httpGet.setURI(uriBuilder.build());

            CloseableHttpResponse response = client.execute(httpGet);
            StatusLine statusLine = response.getStatusLine();
            if (statusLine.getStatusCode() == 200) {
                HttpEntity responseEntity = response.getEntity();
                if (Objects.nonNull(responseEntity)) {
                    String responseBody = EntityUtils.toString(responseEntity, "UTF-8");
                    //log.info("getAllUserLimit10000 responseBody {}", responseBody);
                    return JSONUtils.fromJson(responseBody, new TypeToken<WxUserGetResponse>() {
                    }, null);
                }
            }
        } catch (Exception e) {
            log.error("get user error gzhId {} ", gzhId, e);
        }
        return null;
    }

    public WxUserGetResponse getAllUserLimit10000(String gzhId, String nextOpenId) {
        try {

            String accessToken = wxAccessTokenRemoteService.getAccessToken(gzhId);
            if (StringUtils.isBlank(accessToken)) {
                log.error("gzh {} access_token not config", gzhId);
                return null;
            }

            URIBuilder uriBuilder = new URIBuilder("https://api.weixin.qq.com/cgi-bin/user/get");
            uriBuilder.setParameter("gzhId", gzhId); // 假设需要传递gzhId参数
            uriBuilder.setParameter("access_token", accessToken);
            uriBuilder.setParameter("next_openid", nextOpenId);
            HttpGet httpGet = new HttpGet();
            httpGet.setURI(uriBuilder.build());

            CloseableHttpResponse response = client.execute(httpGet);
            StatusLine statusLine = response.getStatusLine();
            if (statusLine.getStatusCode() == 200) {
                HttpEntity responseEntity = response.getEntity();
                if (Objects.nonNull(responseEntity)) {
                    String responseBody = EntityUtils.toString(responseEntity, "UTF-8");
                    return JSONUtils.fromJson(responseBody, new TypeToken<WxUserGetResponse>() {
                    }, null);
                }
            }
        } catch (Exception e) {
            log.error("get user error gzhId={} nextOpenId={} ", gzhId, nextOpenId, e);
        }
        return null;
    }

}
