package com.tzld.longarticle.recommend.server.service.score.strategy;

import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.model.ContentHisPublishArticle;
import com.tzld.longarticle.recommend.server.service.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.AccountIndexAvgViewCountService;
import com.tzld.longarticle.recommend.server.service.score.Score;
import com.tzld.longarticle.recommend.server.service.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.util.MathUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
@Slf4j
public class ViewCountRateStrategy implements ScoreStrategy {

    @Autowired
    AccountIndexAvgViewCountService accountIndexAvgViewCountService;
    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;

    @Override
    public List<Score> score(ScoreParam param) {
        List<Score> scores = new ArrayList<>();
        for (Content content : param.getContents()) {
            String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());
            for (int i = 0; i < contentPools.length; i++) {
                if (!contentPools[i].equals(content.getContentPoolType())) {
                    continue;
                }
                double avgViewCountPos = accountIndexAvgViewCountService.getAvgReadCountByDB(param.getGhId(), i + 1);
                double avgViewCountFirst = accountIndexAvgViewCountService.getAvgReadCountByDB(param.getGhId(), 1);
                double showViewCountSum = 0D;
                double avgViewCountSum = 0D;
                double showViewCountSumFirst = 0D;
                double avgViewCountSumFirst = 0D;
                double showViewCountSumSecond = 0D;
                double avgViewCountSumSecond = 0D;
                for (ContentHisPublishArticle hisItem : content.getHisPublishArticleList()) {
                    if (hisItem.isInnerAccount() && Objects.nonNull(hisItem.getViewCount())
                            && hisItem.getViewCount() > 0 && Objects.nonNull(hisItem.getAvgViewCount())
                            && hisItem.getAvgViewCount() > 0) {
                        if (hisItem.getItemIndex() == 1) {
                            showViewCountSumFirst += hisItem.getViewCount();
                            avgViewCountSumFirst += hisItem.getAvgViewCount();
                        } else if (hisItem.getItemIndex() == 2) {
                            showViewCountSumSecond += hisItem.getViewCount();
                            avgViewCountSumSecond += hisItem.getAvgViewCount();
                        } else {
                            showViewCountSum += hisItem.getViewCount();
                            avgViewCountSum += hisItem.getAvgViewCount();
                        }
                    }
                }
                double viewCountRate = 0D; // 设置默认值
                double minRate = 5D;
                // 如果有头条反馈数据，优先选取头条反馈数据；
                if (showViewCountSumFirst > 0) {
                    showViewCountSum = showViewCountSumFirst;
                    avgViewCountSum = avgViewCountSumFirst;
                    avgViewCountPos = avgViewCountFirst;
                } else if (showViewCountSumSecond > 0) {
                    showViewCountSum = showViewCountSumSecond;
                    avgViewCountSum = avgViewCountSumSecond;
                    avgViewCountPos = avgViewCountFirst;
                    // 如果是大号头条，则降权
                    if (avgViewCountFirst >= 2000 && i == 0) {
                        minRate = 1.001D;
                    }
                }
                if (avgViewCountSum > 0) {
                    viewCountRate = showViewCountSum / avgViewCountSum;
                }
                double viewCountRateW = MathUtils.sigmoid(avgViewCountSum, 0.0005, avgViewCountPos);
                double viewCountRateScore = 0;
                if (viewCountRate > 0) {
                    viewCountRateScore = (Math.min(viewCountRate, minRate) - 1D) * viewCountRateW;
                }
                Score score = new Score();
                score.setStrategy(this);
                score.setContentId(content.getId());
                score.setScore(viewCountRateScore);
                scores.add(score);
                break;
            }
        }
        return scores;
    }
}
